@@ -8,7 +8,7 @@ Perl::Dist::WiX::BuildPerl - 4th generation Win32 Perl distribution builder
 
 =head1 VERSION
 
-This document describes Perl::Dist::WiX::BuildPerl version 1.102.
+This document describes Perl::Dist::WiX::BuildPerl version 1.200.
 
 =head1 DESCRIPTION
 
@@ -18,6 +18,7 @@ build Perl itself.
 =head1 SYNOPSIS
 
 	# This module is not to be used independently.
+	# It provides methods to be called on a Perl::Dist::WiX object.
 
 =head1 INTERFACE
 
@@ -33,29 +34,30 @@ use Storable qw( retrieve );
 use File::Spec::Functions qw(
   catdir catfile catpath tmpdir splitpath rel2abs curdir
 );
-use Module::CoreList 2.18 qw();
+use Module::CoreList 2.29 qw();
+use Perl::Dist::WiX::Asset::Perl qw();
+use Perl::Dist::WiX::Toolchain qw();
+use File::List::Object qw();
 
-# require IO::Handle;
-require Perl::Dist::WiX::Asset::Perl;
-require Perl::Dist::WiX::Toolchain;
-require File::List::Object;
-
-our $VERSION = '1.102002';
+our $VERSION = '1.200';
 $VERSION =~ s/_//sm;
 
 Readonly my %CORE_MODULE_FIX => (
-	'IO::Compress'        => 'IO::Compress::Base',
-	'Filter'              => 'Filter::Util::Call',
-	'autodie'             => 'Fatal',
-	'Pod'                 => 'Pod::Man',
-	'Text'                => 'Text::Tabs',
-	'PathTools'           => 'Cwd',
-	'Scalar::List::Utils' => 'List::Util',
+	'IO::Compress'         => 'IO::Compress::Base',
+	'Filter'               => 'Filter::Util::Call',
+	'Pod'                  => 'Pod::Man',
+	'Text'                 => 'Text::Tabs',
+	'PathTools'            => 'Cwd',
+	'Scalar::List::Utils'  => 'List::Util',
+	'Locale::Constants'    => 'Locale::Codes',
+	'TermReadKey'          => 'Term::ReadKey',
+	'Term::ReadLine::Perl' => 'Term::ReadLine',
+	'libwww::perl'         => 'LWP',
+	'LWP::UserAgent'       => 'LWP',
 );
 
 Readonly my %DIST_TO_MODULE_FIX => (
 	'CGI.pm'               => 'CGI',
-	'Fatal'                => 'autodie',
 	'Locale::Maketext'     => 'Locale-Maketext',
 	'Pod::Man'             => 'Pod',
 	'Text::Tabs'           => 'Text',
@@ -69,19 +71,23 @@ Readonly my %DIST_TO_MODULE_FIX => (
 	'IO::Scalar'           => 'IO::Stringy',
 );
 
-
+# Modules to delay.
 Readonly my @MODULE_DELAY => qw(
   CPANPLUS::Dist::Build
   File::Fetch
   Thread::Queue
 );
 
+
+
 sub _delay_upgrade {
 	my ( $self, $module ) = @_;
 
 	return ( any { $module->id eq $_ } @MODULE_DELAY ) ? 1 : 0;
 }
 
+
+
 sub _module_fix {
 	my ( $self, $module ) = @_;
 
@@ -113,7 +119,9 @@ the default tasklist after the "perl toolchain" is installed.
 
 sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 	my $self = shift;
-	unless ( $self->bin_perl() ) {
+
+	# Check for Perl - we can't do things out of order.
+	if ( not $self->bin_perl() ) {
 		PDWiX->throw(
 			'Cannot install CPAN modules yet, perl is not installed');
 	}
@@ -121,16 +129,18 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 	# Get list of modules to be upgraded.
 	# (The list is saved as a Storable arrayref of CPAN::Module objects.)
 	my $cpan_info_file = $self->_get_cpan_upgrades_list();
-	my $module_info    = retrieve $cpan_info_file;
+	my $module_info    = retrieve($cpan_info_file);
 
+	# Now go through the loop for each module.
 	my $force;
 	my @delayed_modules;
 	require CPAN;
   MODULE:
 
 	for my $module ( @{$module_info} ) {
-		$force = $self->force;
+		$force = $self->force();
 
+		# Skip modules that we want to skip.
 		next MODULE if $self->_skip_upgrade($module);
 
 		# Net::Ping seems to require that a web server be
@@ -148,47 +158,59 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 		}
 
 		# Locale::Maketext::Simple 0.20 has a test bug. Forcing.
-		if ( $module->cpan_file() =~ m{/Locale-Maketext-Simple-0 [.] 20}msx )
+		if (
+			$module->cpan_file() =~ m{/Locale-Maketext-Simple-0 [.] 20}msx )
 		{
 			$self->_install_cpan_module( $module, 1 );
 			next MODULE;
 		}
 
-		# IPC-System-Simple's t\win32.t has the wrong number of tests.
-		if ( $module->cpan_file() =~ m{/IPC-System-Simple-1 [.] 19\d}msx ) {
-			$self->_install_cpan_module( $module, 1 );
-			next MODULE;
-		}
-
 		# Time-HiRes is timing-dependent, of course.
 		if ( $module->cpan_file() =~ m{/Time-HiRes-}msx ) {
 			$self->_install_cpan_module( $module, 1 );
 			next MODULE;
 		}
 
-		# autodie 2.08/2.09 has a failing test (RT#54525).
-		# It's skipped on 5.10.1+, which already has this version.
-		if ( $module->cpan_file() =~ m{/autodie-2 [.] 0 [89]}msx ) {
-			$self->install_distribution(
-				name             => 'PJF/autodie-2.06_01.tar.gz',
-				mod_name         => 'autodie',
-				makefilepl_param => ['INSTALLDIRS=perl'],
-				buildpl_param    => [ '--installdirs', 'core' ],
-				force            => $force,
-			);
-			next MODULE;
-		}
-
+		# Module::Install needs a few more modules.
 		if (    ( $module->cpan_file() =~ m{/Module-Install-\d}msx )
 			and ( $module->cpan_version() > 0.79 ) )
 		{
-
-			# We need a few more modules.
 			$self->install_modules(qw( File::Remove YAML::Tiny ));
 			$self->_install_cpan_module( $module, $force );
 			next MODULE;
 		}
 
+		# There's a problem with extracting these two files, so
+		# upgrading to these versions, instead...
+		if ( $module->cpan_file() =~
+			/Unicode-Collate-0 [.] 53-withoutworldwriteables/msx )
+		{
+			$self->install_distribution(
+				name     => 'SADAHIRO/Unicode-Collate-0.53.tar.gz',
+				mod_name => 'Unicode::Collate',
+				$self->_install_location(1),
+				$force
+				? ( force => 1 )
+				: (),
+			);
+			next MODULE;
+		} ## end if ( $module->cpan_file...)
+
+		if ( $module->cpan_file() =~
+			/Unicode-Normalize-1 [.] 06-withoutworldwriteables/msx )
+		{
+			$self->install_distribution(
+				name     => 'SADAHIRO/Unicode-Normalize-1.06.tar.gz',
+				mod_name => 'Unicode::Normalize',
+				$self->_install_location(1),
+				$force
+				? ( force => 1 )
+				: (),
+			);
+			next MODULE;
+		} ## end if ( $module->cpan_file...)
+
+		# Get rid of the old ExtUtils::MakeMaker files.
 		if (    ( $module->cpan_file() =~ m{/ExtUtils-MakeMaker-\d}msx )
 			and ( $module->cpan_version() > 6.50 ) )
 		{
@@ -198,6 +220,7 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 			next MODULE;
 		}
 
+		# The podlators dist needs a few more modules to install on 5.8.9.
 		if (    ( $module->cpan_file() =~ m{/podlators-\d}msx )
 			and ( $module->cpan_version() > 2.00 )
 			and ( $self->perl_version() < 5100 ) )
@@ -207,6 +230,7 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 			next MODULE;
 		}
 
+		# New CGI.pm versions require FCGI.
 		if (    ( $module->cpan_file() =~ m{/CGI [.] pm-\d}msx )
 			and ( $module->cpan_version() > 3.45 ) )
 		{
@@ -215,9 +239,8 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 			next MODULE;
 		}
 
+		# If we're on the "delay this module" list, do so.
 		if ( $self->_delay_upgrade($module) ) {
-
-			# Delay these module until last.
 			unshift @delayed_modules, $module;
 			next MODULE;
 		}
@@ -225,6 +248,7 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 		$self->_install_cpan_module( $module, $force );
 	} ## end for my $module ( @{$module_info...})
 
+	# NOW install delayed modules!
 	for my $module (@delayed_modules) {
 		$self->_install_cpan_module( $module, $force );
 	}
@@ -233,7 +257,7 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 	# Yes, I know that technically it's a core module with
 	# non-core dependencies, and that's ugly. I've just got
 	# to live with it.
-	my $autodie_location = $self->_file(qw(perl lib autodie.pm));
+	my $autodie_location = $self->file(qw(perl lib autodie.pm));
 
 	if ( -e $autodie_location ) {
 		$self->install_modules(qw( Win32::Process IPC::System::Simple ));
@@ -241,8 +265,7 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 
 	# Getting CPANPLUS config file installed if required.
 	my $cpanp_config_location =
-	  $self->_file(qw(perl lib CPANPLUS Config.pm));
-
+	  $self->file(qw(perl lib CPANPLUS Config.pm));
 	if ( -e $cpanp_config_location ) {
 
 		$self->trace_line( 1,
@@ -253,9 +276,10 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 			{ dist => $self, } );
 	}
 
+	# Install dev version of CPAN if we haven't already.
 	if ( not $self->fragment_exists('CPAN') ) {
 		$self->install_distribution(
-			name             => 'ANDK/CPAN-1.94_54.tar.gz',
+			name             => 'ANDK/CPAN-1.94_56.tar.gz',
 			mod_name         => 'CPAN',
 			makefilepl_param => ['INSTALLDIRS=perl'],
 			buildpl_param    => [ '--installdirs', 'core' ],
@@ -268,11 +292,10 @@ sub install_cpan_upgrades { ## no critic(ProhibitExcessComplexity)
 sub _get_cpan_upgrades_list {
 	my $self = shift;
 
-	# Generate the CPAN installation script
+	# Get the CPAN url.
 	my $url = $self->cpan()->as_string();
 
-	$url =~ s{file:///C:/}{file://C:/}msx;
-
+	# Generate the CPAN installation script
 	my $cpan_string = <<"END_PERL";
 print "Loading CPAN...\\n";
 use CPAN;
@@ -344,7 +367,6 @@ unless (%need) {
 }
 	
 END_PERL
-
 	my $cpan_info_file = catfile( $self->output_dir(), 'cpan.info' );
 	$cpan_string .= <<"END_PERL";
 nstore \\\@toget, '$cpan_info_file';
@@ -353,7 +375,7 @@ print "Completed collecting information on all modules\\n";
 exit 0;
 END_PERL
 
-	# Dump the CPAN script to a temp file and execute
+	# Dump the CPAN script to a temp file and execute.
 	$self->trace_line( 1, "Running upgrade of all modules\n" );
 	my $cpan_file = catfile( $self->build_dir(), 'cpan_string.pl' );
   SCOPE: {
@@ -365,7 +387,7 @@ END_PERL
 		close $CPAN_FILE
 		  or PDWiX->throw("CPAN script close failed: $OS_ERROR");
 	}
-	$self->_run3( $self->bin_perl(), $cpan_file )
+	$self->execute_perl($cpan_file)
 	  or PDWiX->throw('CPAN script execution failed');
 	PDWiX->throw('Failure detected during cpan upgrade, stopping')
 	  if $CHILD_ERROR;
@@ -375,6 +397,8 @@ END_PERL
 
 sub _install_location {
 	my ( $self, $core ) = @_;
+
+	# Return the correct location information.
 	my $portable = $self->portable();
 	if ($core) {
 		return (
@@ -394,8 +418,12 @@ sub _install_location {
 	}
 } ## end sub _install_location
 
+
+
 sub _install_cpan_module {
 	my ( $self, $module, $force ) = @_;
+
+	# Collect information.
 	$force = $force or $self->force();
 	my $perl_version = $self->perl_version_literal();
 	my $module_id    = $self->_module_fix( $module->id() );
@@ -405,6 +433,15 @@ sub _install_cpan_module {
 	  exists $Module::CoreList::version{ $perl_version }{ $module_id }
 	  ? 1
 	  : 0;
+
+	# Override core determination for this module.
+	# (It's not in core, but other modules that this distribution
+	# installs are)
+	if ('Locale::Codes' eq $module_id) {
+		$core = 1;
+	}
+
+	# Actually do the installation.
 	$self->install_distribution(
 		name     => $module_file,
 		mod_name => $module_id,
@@ -417,10 +454,12 @@ sub _install_cpan_module {
 	return 1;
 } ## end sub _install_cpan_module
 
+
+
 sub _skip_upgrade {
 	my ( $self, $module ) = @_;
 
-	# DON'T try to install Perl.
+	# DON'T try to install Perl at this point!
 	return 1 if $module->cpan_file() =~ m{/perl-5 [.]}msx;
 
 	# DON'T try to install Term::ReadKey, we
@@ -437,31 +476,20 @@ sub _skip_upgrade {
 	# If the ID is ExtUtils::MakeMaker, we've already installed it.
 	# There were some files gotten rid of after 6.50, so
 	# install_cpan_upgrades thinks that it needs to upgrade
-	# those files using it.
-
-	# DON'T try to install ExtUtils::CBuilder, we
-	# already upgraded it as far as we can. RT#54728.
-	return 1 if $module->cpan_file() =~ m{/ExtUtils-CBuilder-0 [.] 2701}msx;
+	# those files using it when using perl versions that
+	# still had those files.
 
 	# This code is in here for safety as of yet.
 	return 1 if $module->cpan_file() =~ m{/ExtUtils-MakeMaker-6 [.] 50}msx;
 
-	# Skip Test-Harness 3.19, waiting on RT#53912.
-	return 1 if $module->cpan_file() =~ m{/Test-Harness-3 [.] 19}msx;
-
 	# Skip B::C, it does not install on 5.8.9.
 	return 1 if $module->cpan_file() =~ m{/B-C-1 [.]}msx;
 
-	# Skip autodie 2.08 and 2.09, waiting on RT#54525.
-	if (($module->cpan_file() =~ m{/autodie-2 [.] 0 [89]}msx) and
-	    (($self->perl_version() eq '5101') or 
-		 ($self->perl_version() eq 'git'))) {
-		return 1;
-	}
-
 	return 0;
 } ## end sub _skip_upgrade
 
+
+
 #####################################################################
 # Perl installation support
 
@@ -471,7 +499,7 @@ The C<install_perl> method is a minimal routine provided to call the
 correct L<install_perl_*|/install_perl_*> routine for the version of perl 
 being created.
 
-Returns true (technically, the object that called it), or throws an exception.
+Returns true or throws an exception.
 
 This is recommended to be used as a task in the tasklist, and is in 
 the default tasklist after the "c toolchain" is installed.
@@ -480,22 +508,60 @@ the default tasklist after the "c toolchain" is installed.
 
 # Just hand off to the larger set of Perl install methods.
 sub install_perl {
-	my $self                = shift;
+	my $self = shift;
+
+	# Check for the method, then call it.
 	my $install_perl_method = 'install_perl_' . $self->perl_version();
-	unless ( $self->can($install_perl_method) ) {
+	if ( not $self->can($install_perl_method) ) {
 		PDWiX->throw(
 			"Cannot generate perl, missing $install_perl_method method in "
 			  . ref $self );
 	}
 	$self->$install_perl_method(@_);
 
+	# Add the perllocal.pod to the perl fragment.
 	$self->add_to_fragment( 'perl',
-		[ $self->_file(qw(perl lib perllocal.pod)) ] );
+		[ $self->file(qw(perl lib perllocal.pod)) ] );
 
 	return 1;
 } ## end sub install_perl
 
-=head2 install_perl_* (* = git, 589, 5100, or 5101)
+
+
+sub _create_perl_toolchain {
+	my $self = shift;
+
+	# Prefetch and predelegate the toolchain so that it
+	# fails early if there's a problem
+	$self->trace_line( 1, "Pregenerating toolchain...\n" );
+	my $toolchain = Perl::Dist::WiX::Toolchain->new(
+		perl_version => $self->perl_version_literal(),
+		cpan         => $self->cpan()->as_string(),
+		bits         => $self->bits(),
+	) or PDWiX->throw('Failed to resolve toolchain modules');
+	if ( not eval { $toolchain->delegate(); 1; } ) {
+		PDWiX::Caught->throw(
+			message => 'Delegation error occured',
+			info    => defined($EVAL_ERROR) ? $EVAL_ERROR : 'Unknown error',
+		);
+	}
+	if ( defined $toolchain->get_error() ) {
+		PDWiX::Caught->throw(
+			message => 'Failed to generate toolchain distributions',
+			info    => $toolchain->get_error() );
+	}
+
+	$self->_set_toolchain($toolchain);
+
+	# Make the perl directory if it hasn't been made already.
+	$self->make_path( $self->dir('perl') );
+
+	return $toolchain;
+} ## end sub _create_perl_toolchain
+
+
+
+=head2 install_perl_* (* = git, 589, 5100, 5101, or 5120)
 
 	$self->install_perl_5100;
 
@@ -531,7 +597,7 @@ A short summary of the process would be that it downloads or otherwise
 fetches the named package, unpacks it, copies out any license files from
 the source code, then tweaks the Win32 makefile to point to the specific
 build directory, and then runs make/make test/make install. It also
-registers some environment variables for addition to the Inno Setup
+registers some environment variables for addition to the installer
 script.
 
 It is normally called directly by C<install_perl_*> rather than
@@ -544,44 +610,22 @@ error.
 
 =cut
 
+
+
 #####################################################################
 # Perl 5.8.9 Support
 
 sub install_perl_589 {
 	my $self = shift;
 
-	# Prefetch and predelegate the toolchain so that it
-	# fails early if there's a problem
-	$self->trace_line( 1, "Pregenerating toolchain...\n" );
-	my $toolchain = Perl::Dist::WiX::Toolchain->new(
-		perl_version => $self->perl_version_literal(),
-		cpan         => $self->cpan()->as_string()
-	) or PDWiX->throw('Failed to resolve toolchain modules');
-	unless ( eval { $toolchain->delegate(); 1; } ) {
-		PDWiX::Caught->throw(
-			message => 'Delegation error occured',
-			info    => defined($EVAL_ERROR) ? $EVAL_ERROR : 'Unknown error',
-		);
-	}
-	if ( defined $toolchain->get_error() ) {
-		PDWiX::Caught->throw(
-			message => 'Failed to generate toolchain distributions',
-			info    => $toolchain->get_error() );
-	}
+	# Get the information required for Perl's toolchain.
+	my $toolchain = $self->_create_perl_toolchain();
 
-	$self->_set_toolchain($toolchain);
-
-	# Make the perl directory if it hasn't been made alreafy.
-	$self->_make_path( $self->_dir('perl') );
-
-	# Install the main perl distributions
+	# Install the main perl distribution.
 	$self->install_perl_bin(
-		name       => 'perl',
-		url        => 'http://strawberryperl.com/package/perl-5.8.9.tar.gz',
-		unpack_to  => 'perl',
-		install_to => 'perl',
-		toolchain  => $toolchain,
-		patch      => [ qw{
+		url       => 'http://strawberryperl.com/package/perl-5.8.9.tar.gz',
+		toolchain => $toolchain,
+		patch     => [ qw{
 			  lib/CPAN/Config.pm
 			  win32/config.gc
 			  win32/config_sh.PL
@@ -598,68 +642,52 @@ sub install_perl_589 {
 	return 1;
 } ## end sub install_perl_589
 
+
+
 sub install_perl_bin {
 	my $self = shift;
-	my $perl = Perl::Dist::WiX::Asset::Perl->new(
-		parent => $self,
-		force  => $self->forceperl() || $self->force(),
-		@_,
-	);
 
-	unless ( $self->bin_make() ) {
+	# Check for an error in the object.
+	if ( not $self->bin_make() ) {
 		PDWiX->throw('Cannot build Perl yet, no bin_make defined');
 	}
 
+	# Install perl.
+	my $perl = Perl::Dist::WiX::Asset::Perl->new(
+		parent => $self,
+		@_,
+	);
 	$perl->install();
 
 	# Should have a perl to use now.
-	$self->_set_bin_perl( $self->_file(qw/perl bin perl.exe/) );
+	$self->_set_bin_perl( $self->file(qw/perl bin perl.exe/) );
+
+	# Create the site/bin path so we can add it to the PATH.
+	$self->make_path( catdir( $self->image_dir(), qw(perl site bin) ) );
 
 	# Add to the environment variables
+	$self->add_path( 'perl', 'site', 'bin' );
 	$self->add_path( 'perl', 'bin' );
 
 	return 1;
 } ## end sub install_perl_bin
 
 
+
 #####################################################################
-# Perl 5.10.0 Support
+# Perl 5.10.x Support
 
 sub install_perl_5100 {
 	my $self = shift;
 
-	# Prefetch and predelegate the toolchain so that it
-	# fails early if there's a problem
-	$self->trace_line( 1, "Pregenerating toolchain...\n" );
-	my $toolchain = Perl::Dist::WiX::Toolchain->new(
-		perl_version => $self->perl_version_literal(),
-		cpan         => $self->cpan()->as_string()
-	) or PDWiX->throw('Failed to resolve toolchain modules');
-	unless ( eval { $toolchain->delegate(); 1; } ) {
-		PDWiX::Caught->throw(
-			message => 'Delegation error occured',
-			info    => defined($EVAL_ERROR) ? $EVAL_ERROR : 'Unknown error',
-		);
-	}
-	if ( defined $toolchain->get_error() ) {
-		PDWiX::Caught->throw(
-			message => 'Failed to generate toolchain distributions',
-			info    => $toolchain->get_error() );
-	}
-
-	$self->_set_toolchain($toolchain);
-
-	# Make the perl directory if it hasn't been made already.
-	$self->_make_path( $self->_dir('perl') );
+	# Get the information required for Perl's toolchain.
+	my $toolchain = $self->_create_perl_toolchain();
 
 	# Install the main binary
 	$self->install_perl_bin(
-		name      => 'perl',
 		url       => 'http://strawberryperl.com/package/perl-5.10.0.tar.gz',
-		unpack_to => 'perl',
-		install_to => 'perl',
-		toolchain  => $toolchain,
-		patch      => [ qw{
+		toolchain => $toolchain,
+		patch     => [ qw{
 			  lib/ExtUtils/Command.pm
 			  lib/CPAN/Config.pm
 			  win32/config.gc
@@ -678,41 +706,18 @@ sub install_perl_5100 {
 } ## end sub install_perl_5100
 
 
+
 sub install_perl_5101 {
 	my $self = shift;
 
-	# Prefetch and predelegate the toolchain so that it
-	# fails early if there's a problem
-	$self->trace_line( 1, "Pregenerating toolchain...\n" );
-	my $toolchain = Perl::Dist::WiX::Toolchain->new(
-		perl_version => $self->perl_version_literal,
-		cpan         => $self->cpan->as_string
-	) or PDWiX->throw('Failed to resolve toolchain modules');
-	unless ( eval { $toolchain->delegate; 1; } ) {
-		PDWiX::Caught->throw(
-			message => 'Delegation error occured',
-			info    => defined($EVAL_ERROR) ? $EVAL_ERROR : 'Unknown error',
-		);
-	}
-	if ( defined $toolchain->get_error() ) {
-		PDWiX::Caught->throw(
-			message => 'Failed to generate toolchain distributions',
-			info    => $toolchain->get_error() );
-	}
-
-	$self->_set_toolchain($toolchain);
-
-	# Make the perl directory if it hasn't been made already.
-	$self->_make_path( $self->_dir('perl') );
+	# Get the information required for Perl's toolchain.
+	my $toolchain = $self->_create_perl_toolchain();
 
 	# Install the main binary
 	$self->install_perl_bin(
-		name      => 'perl',
 		url       => 'http://strawberryperl.com/package/perl-5.10.1.tar.gz',
-		unpack_to => 'perl',
-		install_to => 'perl',
-		toolchain  => $toolchain,
-		patch      => [ qw{
+		toolchain => $toolchain,
+		patch     => [ qw{
 			  lib/CPAN/Config.pm
 			  win32/config.gc
 			  win32/config_sh.PL
@@ -729,44 +734,60 @@ sub install_perl_5101 {
 	return 1;
 } ## end sub install_perl_5101
 
+
+
+#####################################################################
+# Perl 5.12.0 Support
+
+sub install_perl_5120 {
+	my $self = shift;
+
+	# Get the information required for Perl's toolchain.
+	my $toolchain = $self->_create_perl_toolchain();
+
+	# Install the main binary
+	$self->install_perl_bin(
+		url => 'http://strawberryperl.com/package/perl-5.12.0.tar.bz2',
+		toolchain => $toolchain,
+		patch     => [ qw{
+			  lib/CPAN/Config.pm
+			  win32/config.gc
+			  win32/config.gc64nox
+			  win32/config_sh.PL
+			  win32/config_H.gc
+			  win32/config_H.gc64nox
+			  }
+		],
+		license => {
+			'perl-5.12.0/Readme'   => 'perl/Readme',
+			'perl-5.12.0/Artistic' => 'perl/Artistic',
+			'perl-5.12.0/Copying'  => 'perl/Copying',
+		},
+	);
+
+	return 1;
+} ## end sub install_perl_5120
+
+
+
+#####################################################################
+# Git checkout support
+
 sub install_perl_git {
 	my $self = shift;
 
-	# Prefetch and predelegate the toolchain so that it
-	# fails early if there's a problem
-	$self->trace_line( 1, "Pregenerating toolchain...\n" );
-	my $toolchain = Perl::Dist::WiX::Toolchain->new(
-		perl_version => $self->perl_version_literal,
-		cpan         => $self->cpan->as_string
-	) or PDWiX->throw('Failed to resolve toolchain modules');
-	unless ( eval { $toolchain->delegate; 1; } ) {
-		PDWiX::Caught->throw(
-			message => 'Delegation error occured',
-			info    => defined($EVAL_ERROR) ? $EVAL_ERROR : 'Unknown error',
-		);
-	}
-	if ( defined $toolchain->get_error() ) {
-		PDWiX::Caught->throw(
-			message => 'Failed to generate toolchain distributions',
-			info    => $toolchain->get_error() );
-	}
-
-	$self->_set_toolchain($toolchain);
-
-	# Make the perl directory if it hasn't been made already.
-	$self->_make_path( $self->_dir('perl') );
+	# Get the information required for Perl's toolchain.
+	my $toolchain = $self->_create_perl_toolchain();
 
+	# Get where the git checkout is.
 	my $checkout = $self->git_checkout();
 
 	# Install the main binary
 	$self->install_perl_bin(
-		name       => 'perl',
-		url        => URI::file->new($checkout)->as_string(),
-		file       => $checkout,
-		unpack_to  => 'perl',
-		install_to => 'perl',
-		toolchain  => $toolchain,
-		patch      => [ qw{
+		url       => URI::file->new($checkout)->as_string(),
+		file      => $checkout,
+		toolchain => $toolchain,
+		patch     => [ qw{
 			  lib/CPAN/Config.pm
 			  win32/config.gc
 			  win32/config_sh.PL
@@ -784,10 +805,11 @@ sub install_perl_git {
 	return 1;
 } ## end sub install_perl_git
 
+
+
 #####################################################################
 # Perl Toolchain Support
 
-
 =head2 install_perl_toolchain
 
 The C<install_perl_toolchain> method is a routine provided to install the
@@ -802,16 +824,16 @@ the default tasklist after the perl interpreter is installed.
 =cut
 
 sub install_perl_toolchain {
-	my $self      = shift;
-	my $toolchain = $self->_get_toolchain();
+	my $self = shift;
 
+	# Retrieves and verifies the toolchain.
+	my $toolchain = $self->_get_toolchain();
 	if ( 0 == $toolchain->dist_count() ) {
 		PDWiX->throw('Toolchain did not get collected');
 	}
 
-	my ( $core, $module_id );
-
 	# Install the toolchain dists
+	my ( $core, $module_id );
 	foreach my $dist ( $toolchain->get_dists() ) {
 		my $automated_testing = 0;
 		my $release_testing   = 0;
@@ -845,29 +867,14 @@ sub install_perl_toolchain {
 			# Upgrading to this version, instead...
 			$dist = 'STSI/TermReadKey-2.30.01.tar.gz';
 		}
-		if ( $dist =~ /Test-Harness-3 [.] 20/msx ) {
-
-			# 3.20 has a test bug... waiting for RT#54028
-			$force = 1;
-		}
 		if ( $dist =~ /CPAN-1 [.] 9402/msx ) {
 
 			# 1.9402 fails its tests... ANDK says it's a test bug.
 			# Alias agrees that we include 1.94_51 because of the fix
 			# for the Win32 file:// bug.
-			$dist  = 'ANDK/CPAN-1.94_53.tar.gz';
+			$dist  = 'ANDK/CPAN-1.94_56.tar.gz';
 			$force = 1;
 		}
-		if ( $dist =~ /ExtUtils-ParseXS-2[.]20(?:02)?[.]tar[.]gz/msx ) {
-
-			# 2.20 and 2.2002 are buggy on 5.8.9.
-			$dist = 'DAGOLDEN/ExtUtils-ParseXS-2.20_05.tar.gz';
-		}
-		if ( $dist =~ /ExtUtils-CBuilder-0[.]2701[.]tar[.]gz/msx ) {
-
-			# 0.2701 does not pass tests on 5.10.1. RT#54728.
-			$dist = 'DAGOLDEN/ExtUtils-CBuilder-0.27.tar.gz';
-		}
 		if ( $dist =~ /Win32API-Registry-0 [.] 31/msx ) {
 
 			# 0.31 does not include a Makefile.PL.
@@ -880,15 +887,19 @@ sub install_perl_toolchain {
 		}
 		if ( $dist =~ /Win32API-Registry-/msx ) {
 
-	   # This module needs forced on Vista (and probably 2008/Win7 as well).
+			# This module needs forced on Vista
+			# (and probably 2008/Win7 as well).
 			$force = 1;
 		}
 		if ( $dist =~ /Win32-TieRegistry-/msx ) {
 
-	   # This module needs forced on Vista (and probably 2008/Win7 as well).
+			# This module needs forced on Vista
+			# (and probably 2008/Win7 as well).
 			$force = 1;
 		}
 
+		# Actually DO the installation, now
+		# that we've got the information we need.
 		$module_id = $self->_module_fix( $self->_name_to_module($dist) );
 		$core =
 		  exists $Module::CoreList::version{ $self->perl_version_literal() }
@@ -909,12 +920,15 @@ sub install_perl_toolchain {
 	return $self;
 } ## end sub install_perl_toolchain
 
+
+
 sub _name_to_module {
 	my $self = shift;
 	my $dist = shift;
 
+	# Convert a distribution name with dashes to
+	# a module name with double colons.
 	$self->trace_line( 3, "Trying to get module name out of $dist\n" );
-
 #<<<
 	my ( $module ) = $dist =~ m{\A  # Start the string...
 					[A-Za-z/]*      # With a string of letters and slashes
@@ -928,8 +942,6 @@ sub _name_to_module {
 	return $module;
 } ## end sub _name_to_module
 
-
-
 1;
 
 __END__
@@ -960,7 +972,7 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
-L<Perl::Dist|Perl::Dist>, L<Perl::Dist::WiX|Perl::Dist::WiX>, 
+L<Perl::Dist::WiX|Perl::Dist::WiX>, 
 L<http://ali.as/>, L<http://csjewell.comyr.com/perl/>
 
 =head1 COPYRIGHT AND LICENSE
